import logging
from unittest.mock import MagicMock, patch

from django.test import TestCase

import cra_helper
from cra_helper.templatetags import cra_helper_tags

# Quiet down logging
logging.disable(logging.CRITICAL)


class TestCRAHelperTags(TestCase):
    def test_converts_simple_dict_to_sanitized_json(self):
        props = {
            'foo': 'bar',
            'fizz': 'buzz',
        }
        output = '{"foo": "bar", "fizz": "buzz"}'
        self.assertEqual(cra_helper_tags.json(props), output)

    def test_converts_complex_dict_to_sanitized_json(self):
        props = {
            'foo': 'bar',
            'fizz': {
                'buzz': [1, 'b', 3, 'd'],
            },
        }
        output = '{"foo": "bar", "fizz": {"buzz": [1, "b", 3, "d"]}}'
        self.assertEqual(cra_helper_tags.json(props), output)



import os

from django.core.files.storage import FileSystemStorage
from django.core.checks import Error


class SingleFileStorage(FileSystemStorage):
    '''
    This class allows for a reference to an individual file to be held while the staticfiles
    module attempts to collect static files for deployment.

    To use it, just pass an absolute path to the desired file as `location`
    '''
    def __init__(self, location=None, base_url=None, file_permissions_mode=None,
                 directory_permissions_mode=None):
        self._file_path = ''
        self._file_name = ''

        if os.path.isfile(location):
            # Separate the path from the filename
            self._file_path, self._file_name = os.path.split(location)
        else:
            # This class won't work if `location` doesn't point to an actual file
            raise IOError('Path should point to an existing file')

        # Pass in the folder so this acts like a typical FileSystemStorage that needs a path
        super().__init__(
            self._file_path,
            base_url,
            file_permissions_mode,
            directory_permissions_mode
        )

    def listdir(self, path):
        '''
        Return the file name as the sole file that needs to be collected from the specified path
        '''
        return [], [self._file_name]



import logging
from unittest.mock import MagicMock, patch

from django.test import TestCase
from django.conf import settings
from django.core.servers.basehttp import get_internal_wsgi_application
from django.http import Http404

import cra_helper
from cra_helper.handlers import CRAStaticFilesHandler


# Quiet down logging
logging.disable(logging.CRITICAL)


class TestCRAStaticFilesHandler(TestCase):
    def setUp(self):
        self.handler = CRAStaticFilesHandler(get_internal_wsgi_application())
        self._request = MagicMock(path='/fizz/buzz.jpg')
        setattr(settings, 'DEBUG', True)

    def test_should_serve_when_file_exists(self):
        self.handler._should_handle = MagicMock(return_value=True)
        self.handler.serve = MagicMock()

        self.handler.get_response(self._request)

        self.assertTrue(self.handler.serve.called)

    @patch('cra_helper.handlers.redirect')
    def test_should_redirect_when_file_404s(self, mock_redirect):
        self.handler._should_handle = MagicMock(return_value=True)
        self.handler.serve = MagicMock(side_effect=Http404)

        self.handler.get_response(self._request)

        self.assertTrue(mock_redirect.called)

    @patch('cra_helper.handlers.redirect')
    def test_should_redirect_back_to_cra_liveserver(self, mock_redirect):
        cra_helper.handlers.CRA_URL = 'http://foo.bar'
        self.handler._should_handle = MagicMock(return_value=True)
        self.handler.serve = MagicMock(side_effect=Http404)

        self.handler.get_response(self._request)

        mock_redirect.assert_called_with('http://foo.bar/fizz/buzz.jpg')

    @patch('cra_helper.handlers.redirect')
    def test_should_not_redirect_when_file_404s_in_prod(self, mock_redirect):
        setattr(settings, 'DEBUG', False)
        self.handler._should_handle = MagicMock(return_value=True)
        self.handler.serve = MagicMock(side_effect=Http404)

        self.handler.get_response(self._request)

        self.assertFalse(mock_redirect.called)



import os
import logging
from unittest.mock import MagicMock, patch, mock_open

from django.test import TestCase
from django.conf import settings

import cra_helper
from cra_helper import asset_manifest

# Quiet down logging
logging.disable(logging.CRITICAL)

server_url = 'http://foo.bar:9999'

# The originally-supported version of the asset manifest, used through react-scripts v2.1.8
open_mock_v2 = mock_open(
    read_data='''{
        "main.css": "/static/css/main.80e572c9.chunk.css",
        "main.js": "/static/js/main.ef0788cc.chunk.js",
        "main.js.map": "/static/js/main.ef0788cc.chunk.js.map",
        "runtime~main.js": "/static/js/runtime~main.a8a9905a.js",
        "runtime~main.js.map": "/static/js/runtime~main.a8a9905a.js.map",
        "static/js/2.597565cd.chunk.js": "/static/js/2.597565cd.chunk.js",
        "static/js/2.597565cd.chunk.js.map": "/static/js/2.597565cd.chunk.js.map",
        "index.html": "/index.html",
        "precache-manifest.06aece505f403b62363ab3795d4669de.js": "/precache-manifest.06aece505f403b62363ab3795d4669de.js",
        "service-worker.js": "/service-worker.js",
        "static/css/main.80e572c9.chunk.css.map": "/static/css/main.80e572c9.chunk.css.map",
        "static/media/logo.svg": "/static/media/logo.25bf045c.svg"
    }'''
)

# A newer version of the asset manifest used in react-scripts v3.0.0 till v3.2.0
open_mock_v3_1_2 = mock_open(
    read_data='''{
        "files": {
            "main.css": "/static/css/main.b100e6da.chunk.css",
            "main.js": "/static/js/main.5745ba44.chunk.js",
            "main.js.map": "/static/js/main.5745ba44.chunk.js.map",
            "runtime-main.js": "/static/js/runtime-main.e5179790.js",
            "runtime-main.js.map": "/static/js/runtime-main.e5179790.js.map",
            "static/js/2.24ceb05b.chunk.js": "/static/js/2.24ceb05b.chunk.js",
            "static/js/2.24ceb05b.chunk.js.map": "/static/js/2.24ceb05b.chunk.js.map",
            "index.html": "/index.html",
            "precache-manifest.ed3a557b9335d6efb0f3f61d4c8baa40.js": "/precache-manifest.ed3a557b9335d6efb0f3f61d4c8baa40.js",
            "service-worker.js": "/service-worker.js",
            "static/css/main.b100e6da.chunk.css.map": "/static/css/main.b100e6da.chunk.css.map",
            "static/media/logo.svg": "/static/media/logo.25bf045c.svg"
        }
    }'''
)

# A newer version of the asset manifest used in react-scripts v3.2+
open_mock_v_3_2_0 = mock_open(
    read_data='''{
        "files": {
            "main.css": "/static/css/main.b100e6da.chunk.css",
            "main.js": "/static/js/main.f245946a.chunk.js",
            "main.js.map": "/static/js/main.f245946a.chunk.js.map",
            "runtime-main.js": "/static/js/runtime-main.7b0131e1.js",
            "runtime-main.js.map": "/static/js/runtime-main.7b0131e1.js.map",
            "static/js/2.7059510f.chunk.js": "/static/js/2.7059510f.chunk.js",
            "static/js/2.7059510f.chunk.js.map": "/static/js/2.7059510f.chunk.js.map",
            "index.html": "/index.html",
            "precache-manifest.68f3de19261ab9bb69c0c6a58c46215e.js": "/precache-manifest.68f3de19261ab9bb69c0c6a58c46215e.js",
            "service-worker.js": "/service-worker.js",
            "static/css/main.b100e6da.chunk.css.map": "/static/css/main.b100e6da.chunk.css.map",
            "static/media/logo.svg": "/static/media/logo.25bf045c.svg"
        },
        "entrypoints": [
            "static/js/runtime-main.7b0131e1.js",
            "static/js/2.7059510f.chunk.js",
            "static/css/main.b100e6da.chunk.css",
            "static/js/main.f245946a.chunk.js"
        ]
    }'''
)


class TestGenerateManifest(TestCase):
    def test_returns_dict(self):
        self.assertIsInstance(asset_manifest.generate_manifest('', ''), dict)

    @patch('cra_helper.asset_manifest.hosted_by_liveserver', return_value=True)
    def test_returns_bundle_url_if_cra_is_running(self, mock_hosted_check):
        with mock_hosted_check:
            self.assertEqual(asset_manifest.generate_manifest(server_url, ''), {
                'bundle_js': [
                    'http://foo.bar:9999/static/js/bundle.js',
                    'http://foo.bar:9999/static/js/0.chunk.js',
                    'http://foo.bar:9999/static/js/1.chunk.js',
                    'http://foo.bar:9999/static/js/main.chunk.js'
                ]
            })

    def test_returns_manifest_paths_when_cra_is_not_running(self):
        with patch('builtins.open', open_mock_v2):
            self.assertEqual(asset_manifest.generate_manifest(server_url, '.'), {
                'main_css': 'css/main.80e572c9.chunk.css',
                'main_js': 'js/main.ef0788cc.chunk.js',
                'main_js_map': 'js/main.ef0788cc.chunk.js.map',
                'runtime_main_js': 'js/runtime~main.a8a9905a.js',
                'runtime_main_js_map': 'js/runtime~main.a8a9905a.js.map',
                'static_css_main_80e572c9_chunk_css_map': 'css/main.80e572c9.chunk.css.map',
                'static_js_2_597565cd_chunk_js': 'js/2.597565cd.chunk.js',
                'static_js_2_597565cd_chunk_js_map': 'js/2.597565cd.chunk.js.map',
                'static_media_logo_svg': 'media/logo.25bf045c.svg'
            })

    def test_checks_static_root_when_manifest_missing_from_cra_build_dir(self):
        # Pretend this folder is STATIC_ROOT to emulate falling back and accessing the
        # collected asset-manifest.json
        _here_path = os.path.dirname(os.path.realpath(__file__))
        setattr(settings, 'BASE_DIR', _here_path)
        setattr(settings, 'STATIC_ROOT', os.path.join(_here_path, 'static'))
        self.assertEqual(asset_manifest.generate_manifest(server_url, 'not_a_real_dir'), {
            'main_js': 'js/main.1234.js',
            'main_css': 'css/main.1234.css',
        })

    def test_returns_empty_dict_if_file_not_found(self):
        open_mock = MagicMock(side_effect=Exception)
        with patch('builtins.open', open_mock):
            self.assertEqual(asset_manifest.generate_manifest(server_url, '.'), {})

    def test_handles_manifest_v_3_1_2(self):
        with patch('builtins.open', open_mock_v3_1_2):
            self.assertEqual(asset_manifest.generate_manifest(server_url, '.'), {
                'main_css': 'css/main.b100e6da.chunk.css',
                'main_js': 'js/main.5745ba44.chunk.js',
                'main_js_map': 'js/main.5745ba44.chunk.js.map',
                'runtime_main_js': 'js/runtime-main.e5179790.js',
                'runtime_main_js_map': 'js/runtime-main.e5179790.js.map',
                'static_css_main_b100e6da_chunk_css_map': 'css/main.b100e6da.chunk.css.map',
                'static_js_2_24ceb05b_chunk_js': 'js/2.24ceb05b.chunk.js',
                'static_js_2_24ceb05b_chunk_js_map': 'js/2.24ceb05b.chunk.js.map',
                'static_media_logo_svg': 'media/logo.25bf045c.svg'
            })

    def test_handles_manifest_v_3_2_0(self):
        self.maxDiff = None
        with patch('builtins.open', open_mock_v_3_2_0):
            self.assertEqual(asset_manifest.generate_manifest(server_url, '.'), {
                'main_css': 'css/main.b100e6da.chunk.css',
                'main_js': 'js/main.f245946a.chunk.js',
                'main_js_map': 'js/main.f245946a.chunk.js.map',
                'runtime_main_js': 'js/runtime-main.7b0131e1.js',
                'runtime_main_js_map': 'js/runtime-main.7b0131e1.js.map',
                'static_css_main_b100e6da_chunk_css_map': 'css/main.b100e6da.chunk.css.map',
                'static_js_2_7059510f_chunk_js': 'js/2.7059510f.chunk.js',
                'static_js_2_7059510f_chunk_js_map': 'js/2.7059510f.chunk.js.map',
                'static_media_logo_svg': 'media/logo.25bf045c.svg',
                'entrypoints': {
                    'css': [
                        'css/main.b100e6da.chunk.css'
                    ],
                    'js': [
                        'js/runtime-main.7b0131e1.js',
                        'js/2.7059510f.chunk.js',
                        'js/main.f245946a.chunk.js'
                    ]
                }
            })



import os
import json
import re

from django.conf import settings

from cra_helper.logging import logger
from cra_helper.server_check import hosted_by_liveserver

_asset_filename = 'asset-manifest.json'


# Sanitize characters that Django's `static` template tag can't handle
def clean_file_key(filename: str) -> str:
    cleaned = filename.replace('.', '_')
    cleaned = cleaned.replace('/', '_')
    cleaned = cleaned.replace('~', '_')
    cleaned = cleaned.replace('-', '_')
    return cleaned

# Create a dictionary of keys that can be passed to the `static` template tag to load CRA assets
# from Django's STATIC_ROOT directory
def generate_manifest(cra_url: str, app_dir: str) -> dict:
    manifest = {}

    # The ability to access this file means the create-react-app liveserver is running
    bundle_path = '{}/static/js/bundle.js'.format(cra_url)
    # Check if Create-React-App live server is up and running
    is_server_live = hosted_by_liveserver(bundle_path)

    # Prepare references to various files frontend
    if is_server_live:
        logger.info('Create-React-App liveserver is running')
        # Earlier versions of CRA included everything in a single bundle.js...
        manifest['bundle_js'] = [
            bundle_path,
        ]

        # ...while more recent versions of CRA use code-splitting and serve additional files
        liveserver_bundles = [
            # These two files will alternate being loaded into the page
            '{}/static/js/0.chunk.js'.format(cra_url),
            '{}/static/js/1.chunk.js'.format(cra_url),
            # This bundle seems to contain some vendor files
            '{}/static/js/main.chunk.js'.format(cra_url)
        ]

        for url in liveserver_bundles:
            if hosted_by_liveserver(url):
                manifest['bundle_js'].append(url)
    else:
        logger.info('Create-React-App liveserver is not running')
        build_dir = os.path.join(app_dir, 'build')

        # Add the CRA static directory to STATICFILES_DIRS so collectstatic can grab files in there
        static_dir = os.path.join(build_dir, 'static')
        settings.STATICFILES_DIRS += [static_dir]

        # CRA generates a JSON file that maps typical filenames to their hashed filenames
        manifest_path = os.path.join(build_dir, _asset_filename)

        # Try to load the JSON manifest from the React build directory first
        try:
            with open(manifest_path) as data_file:
                logger.info('found manifest in React build files')
                data = json.load(data_file)
        except Exception as e:
            # If that doesn't work, try to load it from the Django project's static files directory
            try:
                static_manifest_path = os.path.join(settings.STATIC_ROOT, _asset_filename)
                with open(static_manifest_path) as data_file:
                    logger.info('found manifest in static files')
                    data = json.load(data_file)
            except Exception as e:
                logger.error('can\'t load static asset manifest: {}'.format(e))
                return {}

        # Older versions of Create-React-App (through v2.1.8) had flat manifests. If a "files"
        # property doesn't exist in the manifest JSON, then we're probably working with an older
        # version of CRA.
        if data.get('files') is None:
            manifest_items = data.items()
        else:
            manifest_items = data.get('files').items()

        # Prepare a regex that'll help us detect and remove the leading "/static/" from manifest
        # filepaths. Support relative path prefixes that might also prefix "/static/".
        static_base_path = re.compile(r'^(?:/[\w.-]+)?/static/', re.IGNORECASE)
        for file_key, path in manifest_items:
            # Don't include index.html, serviceWorker.js, etc...
            if static_base_path.match(path):
                # Generate paths relative to our bundled assets
                # Ex: /static/css/main.99358b65.chunk.css => css/main.99358b65.chunk.css
                manifest[clean_file_key(file_key)] = re.sub(static_base_path, '', path)

        # Later versions of Create-React-App (starting with v3.2.0) will tell us which files to
        # load via an "entrypoints" array
        entrypoints = data.get('entrypoints')
        if entrypoints is not None:
            # Prepare to group entrypoint files by extension
            manifest['entrypoints'] = {
                'js': [],
                'css': [],
            };

            # Map manifest files by their path
            mapped_manifest_items = {}
            for file_key, path in manifest_items:
                # Paths in "entrypoints" in asset-manifest.json don't include the relative path
                # that might be set to "homepage" in package.json. Convert the asset file paths in
                # "files" in asset-manifest.json from `/frontend/static/...` to `static/...` so
                # that our truncated manifest file paths generated above can be matched to
                # the "entrypoints" paths
                normalized_path = re.sub(static_base_path, 'static/', path)
                mapped_manifest_items[normalized_path] = clean_file_key(file_key)

            for path in entrypoints:
                rel_static_path = manifest[mapped_manifest_items[path]]

                if path.endswith('.js'):
                    manifest['entrypoints']['js'].append(rel_static_path)
                elif path.endswith('.css'):
                    manifest['entrypoints']['css'].append(rel_static_path)

    return manifest



from django.views.decorators.csrf import csrf_exempt
from proxy.views import proxy_view

from cra_helper import CRA_URL


@csrf_exempt
def proxy_cra_requests(request, path):
  '''
  Proxy various requests sent by Create-React-App projects in dev mode ("npm start"), within
  Django-hosted views, to the Create-React-App liveserver

  Works well with the following re_path definitions in your project's urls.py:

  re_path(r'^sockjs-node/(?P<path>.*)$', proxy_sockjs),
  re_path(r'^__webpack_dev_server__/(?P<path>.*)$', proxy_sockjs),
  '''
  path = request.path
  url = '{}{}'.format(CRA_URL, path)
  return proxy_view(request, url)



'''
JSON filter courtesy of https://gist.github.com/pirate/c18bfe4fd96008ffa0aef25001a2e88f
'''

import bleach
import json as jsonlib

from django import template
from django.utils.safestring import mark_safe

register = template.Library()


@register.filter
def json(value):
    '''
    Sanitize the JSON string using the Bleach HTML tag remover
    '''
    uncleaned = jsonlib.dumps(value)
    clean = bleach.clean(uncleaned)
    return mark_safe(clean)



import os

from django.contrib.staticfiles import utils
from django.contrib.staticfiles.finders import BaseFinder

from cra_helper import CRA_FS_APP_DIR
from cra_helper.storage import SingleFileStorage


class CRAManifestFinder(BaseFinder):
    def __init__(self, app_names=None, *args, **kwargs):
        self._manifest_folder = os.path.join(CRA_FS_APP_DIR, 'build')
        self._manifest_filename = 'asset-manifest.json'
        # Generate an absolute path to the manifest file
        self.manifest_location = os.path.join(self._manifest_folder, self._manifest_filename)
        # Prepare our custom storage handler
        self.manifest_storage = SingleFileStorage(location=self.manifest_location)
        super().__init__(*args, **kwargs)

    def check(self, **kwargs):
        return []

    def find(self, path, all=False):
        matches = []
        if path == self._manifest_filename and os.path.exists(self.manifest_location):
            matches.append(self.manifest_location)
        return matches

    def list(self, ignore_patterns):
        for path in utils.get_files(self.manifest_storage, ignore_patterns):
            yield path, self.manifest_storage



import os

from django.conf import settings

from cra_helper.asset_manifest import generate_manifest

# Variables to help the template determine if it should try loading assets from the CRA live server
_CRA_DEFAULT_PORT = 3000
_CRA_DEFAULT_HOST = 'localhost'
_port = _CRA_DEFAULT_PORT
_host = _CRA_DEFAULT_HOST

# Allow the user to specify the port create-react-app is running on
if hasattr(settings, 'CRA_PORT') and type(settings.CRA_PORT) is int:
    _port = settings.CRA_PORT

# Allow the user to specify the host create-react-app is running on
if hasattr(settings, 'CRA_HOST'):
    _host = settings.CRA_HOST

# The URL the create-react-app liveserver is accessible at
CRA_URL = 'http://{}:{}'.format(_host, _port)

if hasattr(settings, 'CRA_APP_NAME'):
    CRA_APP_NAME = settings.CRA_APP_NAME
else:
    CRA_APP_NAME = 'react'

# The path to the CRA project directory, relative to the Django project's base directory
CRA_FS_APP_DIR = os.path.join(settings.BASE_DIR, CRA_APP_NAME)

# A list of entries in CRA's build bundle's
STATIC_ASSET_MANIFEST = generate_manifest(CRA_URL, CRA_FS_APP_DIR)



from urllib import request, error as url_error

from django.conf import settings

from cra_helper.logging import logger


def hosted_by_liveserver(file_url: str) -> bool:
    # Ignore the server check if we're in production
    if settings.DEBUG:
        try:
            resp = request.urlopen(file_url)
            if resp.status == 200:
                logger.debug('{} is being hosted by liveserver'.format(file_url))
                return True
            else:
                logger.warning('Create-React-App liveserver is up but not serving files')
                return False
        except url_error.URLError as err:
            logger.debug('{} is not being hosted by liveserver'.format(file_url))
            return False
    else:
        logger.debug('Liveserver host check disabled in production')
        return False



import logging
from unittest.mock import MagicMock, patch
from urllib import error as url_error

from django.conf import settings
from django.test import TestCase

from cra_helper import CRA_URL, server_check

# Quiet down logging
logging.disable(logging.CRITICAL)


class TestHostedByLiveserver(TestCase):
    def setUp(self):
        setattr(settings, 'DEBUG', True)

    def test_returns_boolean(self):
        self.assertIsInstance(server_check.hosted_by_liveserver(CRA_URL), bool)

    @patch('urllib.request.urlopen')
    def test_returns_true_when_cra_running_and_debug(self, mock_urlopen):
        mock_urlopen.return_value = MagicMock(status=200)
        self.assertTrue(server_check.hosted_by_liveserver(CRA_URL))

    @patch('urllib.request.urlopen')
    def test_returns_false_when_cra_running_and_production(self, mock_urlopen):
        mock_urlopen.return_value = MagicMock(status=200)
        setattr(settings, 'DEBUG', False)
        self.assertFalse(server_check.hosted_by_liveserver(CRA_URL))

    @patch('urllib.request.urlopen')
    def test_returns_false_when_cra_errors(self, mock_urlopen):
        mock_urlopen.return_value = MagicMock(status=500)
        self.assertFalse(server_check.hosted_by_liveserver(CRA_URL))

    @patch('urllib.request.urlopen')
    def test_returns_false_when_cra_not_running(self, mock_urlopen):
        mock_urlopen.side_effect = url_error.URLError('CRA not running')
        self.assertFalse(server_check.hosted_by_liveserver(CRA_URL))



import logging
from unittest.mock import MagicMock, patch

from django.test import TestCase

import cra_helper
from cra_helper import context_processors

# Quiet down logging
logging.disable(logging.CRITICAL)

request = {}
mock_response = {'foo': 'bar'}


class TestStatic(TestCase):
    def test_returns_dict(self):
        self.assertIsInstance(context_processors.static(request), dict)

    def test_returns_asset_manifest_if_defined(self):
        context_processors.STATIC_ASSET_MANIFEST = mock_response
        self.assertEqual(context_processors.static(request), mock_response)

    def test_returns_empty_dict_if_no_asset_manifest(self):
        context_processors.STATIC_ASSET_MANIFEST = None
        self.assertEqual(context_processors.static(request), {})



from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='django-cra-helper',
    version='1.2.2',
    description='The missing piece of the Django + React puzzle',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/MasterKale/django-cra-helper',
    author='Matthew Miller',
    author_email='matthew@millerti.me',
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
    ],
    keywords='django react create-react-app integrate',
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    install_requires=[
        'bleach>=3.1.4',
        'django-proxy>=1.2.1',
    ],
)



from cra_helper.handlers import CRAStaticFilesHandler
from django.contrib.staticfiles.management.commands.runserver import \
    Command as StaticRunserverCommand
from django.conf import settings


class Command(StaticRunserverCommand):
    help = '''Starts a lightweight Web server for development and also serves static files.
Redirect static file 404s to Create-React-App\'s liveserver.'''

    def get_handler(self, *args, **options):
        '''
        Return the static files serving handler wrapping the default handler,
        if static files should be served. Otherwise return the default handler.
        '''
        handler = super().get_handler(*args, **options)
        use_static_handler = options['use_static_handler']
        insecure_serving = options['insecure_serving']
        if use_static_handler and (settings.DEBUG or insecure_serving):
            return CRAStaticFilesHandler(handler)
        return handler



from django.contrib.staticfiles.handlers import StaticFilesHandler
from django.shortcuts import redirect
from django.conf import settings

from cra_helper import CRA_URL


class CRAStaticFilesHandler(StaticFilesHandler):
    '''
    This file handler redirects static asset 404's to the Create-React-App liveserver
    '''
    def get_response(self, request):
        from django.http import Http404

        if self._should_handle(request.path):
            try:
                # Try to handle the request as usual
                return self.serve(request)
            except Http404 as e:
                # When the file 404's, redirect the request to the CRA liveserver
                if settings.DEBUG:
                    return redirect('{}{}'.format(CRA_URL, request.path))
                # Simply return the 404 outside of DEBUG mode (a.k.a. in production mode)
                return e

        return super().get_response(request)
